#ifndef LIBXMLWRAP_STRING_H
#define LIBXMLWRAP_STRING_H

#include <iterator>

namespace xml
{

class string_error : public std::exception
{
public:
	virtual ~string_error() throw() {}

	virtual const char * what() const throw() { return "xml string error"; }

};

class string
{
public:
	xmlChar char_type;

	string() : buffer_( xmlCharStrdup( "" ) )
	{
		if( !buffer_ )
		{
			throw string_error();
		}
	} 
	string( const xmlChar * utf ) : buffer_( xmlStrdup( utf ) )
	{
		if( !buffer_ )
		{
			throw string_error();
		}
	}
	string( const char * utf ) : buffer_( xmlCharStrdup( utf ) )
	{
		if( !buffer_ )
		{
			throw string_error();
		}
	}
	string( const string & other ) : buffer_( xmlStrdup( other.buffer_ ) )
	{
		if( !buffer_ )
		{
			throw string_error();
		}
	}
	~string()
	{
		xmlFree( buffer_ );
	}
	string & operator = ( const string & other )
	{
		string temp( other );
		swap( temp );
		return *this;
	}

	void swap( string & other )
	{
		std::swap( buffer_, other.buffer_ );
	}

	const char * utf8() const { return reinterpret_cast< const char * >( buffer_ ); }

private:
	xmlChar * buffer_;

	void free_buffer()
	{
		if( buffer_ )
		{
			xmlFree( buffer_ );
			buffer_ = 0;
		}
	}

	friend bool operator == ( const string & s1, const string & s2 );
};

inline bool operator == ( const string & s1, const string & s2 )
{
	return xmlStrEqual( s1.buffer_, s2.buffer_ ) != 0;
}

inline bool operator != ( const string & s1, const string & s2 )
{
	return !( s1 == s2 );
}

} // namespace xml

#endif // LIBXMLWRAP_STRING_H
